@@ -22,7 +22,7 @@
 //! - Not all input formats can translate from a reader, and must consume their
 //!   input from an in-memory slice no matter what.
 //!
-//! [`Handle`] is the main container that allows format detection logic to
+//! [`Handle`] is the main container that allows the format detection logic to
 //! borrow temporary copies of the input, and then allows the selected input
 //! format to take full ownership of the input for translation.
 //!
@@ -35,8 +35,8 @@
 //! any time the format detection process consumes an entire reader, either by
 //! coincidence or by trying to detect a format that requires slice input, all
 //! future use of the input will become slice-based. Alternatively, if format
-//! detection is skipped, the input format will simply take the original reader
-//! without even allocating a capture buffer.
+//! detection is skipped, the selected input format will simply take the
+//! original reader without even allocating a capture buffer.
 
 use std::borrow::Cow;
 use std::io::{self, Cursor, Read, Write};
@@ -52,7 +52,7 @@ pub struct Handle<'i>(Source<'i>);
 /// The private container for the original input a [`Handle`] was created from.
 enum Source<'i> {
   Slice(&'i [u8]),
-  Reader(CaptureReader<Box<dyn Read + 'i>>),
+  Reader(RewindGuard<Box<dyn Read + 'i>>),
 }
 
 impl<'i> Handle<'i> {
@@ -62,8 +62,8 @@ impl<'i> Handle<'i> {
   /// but require all input to be loaded into memory in advance. This may be
   /// inappropriate for an unbounded stream of documents in a format that
   /// supports streaming translation.
-  pub fn from_slice(source: &'i [u8]) -> Handle<'i> {
-    Handle(Source::Slice(source))
+  pub fn from_slice(b: &'i [u8]) -> Handle<'i> {
+    Handle(Source::Slice(b))
   }
 
   /// Creates a handle for an input reader.
@@ -73,11 +73,11 @@ impl<'i> Handle<'i> {
   /// buffering more than one document in memory at a time. When translating
   /// from a format that does not support streaming, xt will buffer the entire
   /// contents of the reader into memory before starting translation.
-  pub fn from_reader<R>(source: R) -> Handle<'i>
+  pub fn from_reader<R>(r: R) -> Handle<'i>
   where
     R: Read + 'i,
   {
-    Handle(Source::Reader(CaptureReader::new(Box::new(source))))
+    Handle(Source::Reader(RewindGuard::new(Box::new(r))))
   }
 
   /// Borrows a temporary reference to the input.
@@ -95,10 +95,11 @@ impl<'i> Handle<'i> {
     match &mut self.0 {
       Source::Slice(b) => Ref::Slice(b),
       Source::Reader(r) => {
+        let r = r.rewind_and_borrow_mut();
         if r.is_source_eof() {
           Ref::Slice(r.captured())
         } else {
-          Ref::Reader(ReaderGuard(r))
+          Ref::Reader(r)
         }
       }
     }
@@ -114,6 +115,8 @@ impl<'i> TryInto<Cow<'i, [u8]>> for Handle<'i> {
     match self.0 {
       Source::Slice(b) => Ok(Cow::Borrowed(b)),
       Source::Reader(r) => {
+        let r = r.rewind_and_take();
+
         if r.is_source_eof() {
           let (cursor, _) = r.into_inner();
           return Ok(Cow::Owned(cursor.into_inner()));
@@ -131,9 +134,9 @@ impl<'i> TryInto<Cow<'i, [u8]>> for Handle<'i> {
 /// A container for owned input obtained by consuming a [`Handle`].
 ///
 /// The kind of `Input` produced from a `Handle` may not correspond directly to
-/// the original `Source`. If a reader input was fully buffered through normal
-/// use of the `Handle`, the `Input` will provide ownership of the buffer, and
-/// the conversion from `Handle` will drop the reader.
+/// the original `Source`. If a reader input was fully buffered through use of
+/// the `Handle`, the `Input` will provide ownership of the buffer, and the
+/// conversion from `Handle` will drop the reader.
 pub(crate) enum Input<'i> {
   Slice(Cow<'i, [u8]>),
   Reader(Box<dyn Read + 'i>),
@@ -144,6 +147,7 @@ impl<'i> From<Handle<'i>> for Input<'i> {
     match handle.0 {
       Source::Slice(b) => Input::Slice(Cow::Borrowed(b)),
       Source::Reader(r) => {
+        let r = r.rewind_and_take();
         let source_eof = r.is_source_eof();
         let (cursor, source) = r.into_inner();
         if source_eof {
@@ -166,7 +170,7 @@ where
   'i: 'h,
 {
   Slice(&'h [u8]),
-  Reader(ReaderGuard<'i, 'h>),
+  Reader(&'h mut CaptureReader<Box<dyn Read + 'i>>),
 }
 
 impl<'i, 'h> Ref<'i, 'h>
@@ -181,8 +185,8 @@ where
     match self {
       Ref::Slice(b) => Ok(b),
       Ref::Reader(r) => {
-        r.0.capture_to_end()?;
-        Ok(r.0.captured())
+        r.capture_to_end()?;
+        Ok(r.captured())
       }
     }
   }
@@ -195,42 +199,46 @@ where
   /// For reader inputs not yet fully buffered, `want_size` represents the
   /// minimum size of the prefix that the call should attempt to produce by
   /// capturing new bytes from the source. The returned prefix may be smaller or
-  /// larger than `want_size` if the reader reaches EOF, more input has already
-  /// been captured, or a larger read is made from the source for efficiency.
+  /// larger than `want_size` if the reader reaches EOF or more input is already
+  /// captured.
   pub(crate) fn prefix(&mut self, want_size: usize) -> io::Result<&[u8]> {
     match self {
       Ref::Slice(b) => Ok(b),
       Ref::Reader(r) => {
-        r.0.capture_to_size(want_size)?;
-        Ok(r.0.captured())
+        r.capture_to_size(want_size)?;
+        Ok(r.captured())
       }
     }
   }
 }
 
-/// Wraps a [`CaptureReader`] to rewind its position to the start when dropped.
+/// A wrapper that forces a [`CaptureReader`] to be rewound prior to use.
 ///
-/// This enables multiple calls to [`Handle::borrow_mut`] to access reader input
-/// without fully consuming it.
-pub(crate) struct ReaderGuard<'i, 'h>(&'h mut CaptureReader<Box<dyn Read + 'i>>)
+/// This is designed to statically prevent bugs that would result from
+/// forgetting to rewind the reader before exposing its contents to consumers.
+struct RewindGuard<R>(CaptureReader<R>)
 where
-  'i: 'h;
+  R: Read;
 
-impl<'i, 'h> Read for ReaderGuard<'i, 'h>
+impl<R> RewindGuard<R>
 where
-  'i: 'h,
+  R: Read,
 {
-  fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
-    self.0.read(buf)
+  /// Returns a new `RewindGuard` containing a new `CaptureReader`.
+  fn new(r: R) -> Self {
+    Self(CaptureReader::new(r))
   }
-}
 
-impl<'i, 'h> Drop for ReaderGuard<'i, 'h>
-where
-  'i: 'h,
-{
-  fn drop(&mut self) {
+  /// Rewinds the `CaptureReader`, then returns a mutable reference.
+  fn rewind_and_borrow_mut(&mut self) -> &mut CaptureReader<R> {
     self.0.rewind();
+    &mut self.0
+  }
+
+  /// Rewinds the `CaptureReader`, then returns it.
+  fn rewind_and_take(mut self) -> CaptureReader<R> {
+    self.0.rewind();
+    self.0
   }
 }
 
@@ -305,12 +313,7 @@ where
   /// The actual number of captured bytes may be less than `size` if the source
   /// reaches EOF before producing `size` bytes.
   fn capture_to_size(&mut self, size: usize) -> io::Result<()> {
-    // This matches the privately defined default size of a BufReader for most
-    // platforms as of this writing. It seems like a reasonable enough lower
-    // bound to prevent us from spending a system call on, say, one byte.
-    const MIN_SIZE: usize = 8 * 1024;
-
-    let needed = std::cmp::max(size, MIN_SIZE).saturating_sub(self.prefix.get_ref().len());
+    let needed = size.saturating_sub(self.prefix.get_ref().len());
     if needed == 0 {
       return Ok(());
     }
@@ -356,17 +359,17 @@ where
     // consumer drive the number and size of reads against the source, making
     // our presence more transparent to both sides. As the smallest consolation,
     // it's worth noting that we only read bytes we know were freshly written,
-    // and do not rely on the original contents of `buf` in any way.
+    // and do not rely on the original contents of `buf` in any way (unless, of
+    // course, the source is broken and lies about how many bytes it read).
     let buf = &mut buf[prefix_size..];
     let source_size = self.source.read(buf)?;
     self.prefix.write_all(&buf[..source_size])?;
 
-    // Finally, mark whether the source has reached EOF. We know that our new
+    // Finally, mark whether the source is at EOF (keeping in mind that it can,
+    // in theory, return more data after an earlier EOF). We know that our new
     // `buf` can't be empty as we return early when `prefix_size == buf.len()`,
     // so a 0 byte read can only indicate EOF.
-    if source_size == 0 {
-      self.source_eof = true;
-    }
+    self.source_eof = source_size == 0;
 
     Ok(prefix_size + source_size)
   }
@@ -374,12 +377,65 @@ where
 
 #[cfg(test)]
 mod tests {
-  use super::CaptureReader;
-  use std::io::{Cursor, Read};
+  use super::{CaptureReader, Handle, Input, Ref};
+  use std::borrow::Cow;
+  use std::io::{self, Cursor, Read};
 
   const DATA: &str = "abcdefghij";
   const HALF: usize = DATA.len() / 2;
 
+  #[test]
+  fn input_borrow_mut_rewind() {
+    let mut handle = Handle::from_reader(DATA.as_bytes());
+    let mut buf = vec![];
+
+    let mut input_ref = handle.borrow_mut();
+    match input_ref {
+      Ref::Slice(_) => unreachable!(),
+      Ref::Reader(ref mut r) => r.take(HALF as u64).read_to_end(&mut buf).unwrap(),
+    };
+    assert_eq!(std::str::from_utf8(&buf), Ok(&DATA[..HALF]));
+    buf.clear();
+
+    // Forgetting the `Ref` must not break the behavior of `borrow_mut`. Yes, I
+    // implemented rewinding based on `Drop` the first time around, which is
+    // obviously wrong.
+    std::mem::forget(input_ref);
+
+    match handle.borrow_mut() {
+      Ref::Slice(_) => unreachable!(),
+      Ref::Reader(r) => r.take(HALF as u64).read_to_end(&mut buf).unwrap(),
+    };
+    assert_eq!(std::str::from_utf8(&buf), Ok(&DATA[..HALF]));
+    buf.clear();
+
+    // If we only consume part of a borrowed reader, we need to reset the reader
+    // before giving ownership away. I made this mistake in my first attempt to
+    // eliminate the `Drop`-based rewind.
+    let mut r = match handle.into() {
+      Input::Slice(_) => unreachable!(),
+      Input::Reader(r) => r,
+    };
+    assert!(matches!(r.read_to_end(&mut buf), Ok(len) if len == DATA.len()));
+    assert_eq!(std::str::from_utf8(&buf), Ok(DATA));
+  }
+
+  #[test]
+  fn input_into_cow() {
+    let mut handle = Handle::from_reader(DATA.as_bytes());
+
+    match handle.borrow_mut() {
+      Ref::Slice(_) => unreachable!(),
+      Ref::Reader(r) => io::copy(&mut r.take(HALF as u64), &mut io::sink()).unwrap(),
+    };
+
+    // Similar to above (though not a mistake I actually made). If we only
+    // consume part of a borrowed reader, turning the input into a slice should
+    // still produce the full input.
+    let buf: Cow<'_, [u8]> = handle.try_into().unwrap();
+    assert_eq!(std::str::from_utf8(&buf), Ok(DATA));
+  }
+
   #[test]
   fn rewindable_reader_straight_read() {
     let mut r = CaptureReader::new(Cursor::new(String::from(DATA)));
@@ -427,10 +483,7 @@ mod tests {
   fn rewindable_reader_capture_up_to() {
     let mut r = CaptureReader::new(Cursor::new(String::from(DATA)));
     assert!(matches!(r.capture_to_size(HALF), Ok(_)));
-
-    // We expect the reader to go all the way to its MIN_SIZE for such a small
-    // request.
-    assert_eq!(std::str::from_utf8(r.captured()), Ok(DATA));
-    assert!(r.is_source_eof());
+    assert_eq!(std::str::from_utf8(r.captured()), Ok(&DATA[..HALF]));
+    assert!(!r.is_source_eof());
   }
 }
