@@ -1,4 +1,4 @@
-//! Functions and types to support translation between Serde data formats.
+//! Translation between Serde data formats.
 //!
 //! xt's transcoder is somewhat inspired by the [`serde_transcode`] crate
 //! advertised in the Serde documentation. However, its implementation has
@@ -257,18 +257,15 @@ impl<P, E> Transcoder<P, E> {
     }
   }
 
-  fn capture_source(&self, source: ErrorSource) {
-    self.source.set(source)
+  fn capture_error(&self, source: ErrorSource, error: Option<E>) {
+    self.source.set(source);
+    self.error.set(error);
   }
 
   fn error_source(&self) -> ErrorSource {
     self.source.get()
   }
 
-  fn capture_error(&self, error: Option<E>) {
-    self.error.set(error)
-  }
-
   fn into_error(self) -> Option<E> {
     self.error.into_inner()
   }
@@ -276,18 +273,17 @@ impl<P, E> Transcoder<P, E> {
 
 /// Handles the capture and mapping of serializer errors in visitor methods that
 /// only need to forward simple values.
-fn forward_value<S, E, F>(t: &Transcoder<S, S::Error>, f: F) -> Result<S::Ok, E>
+fn forward_value<S, E, F>(t: &Transcoder<S, S::Error>, op: F) -> Result<S::Ok, E>
 where
   S: Serializer,
   E: de::Error,
   F: FnOnce(S) -> Result<S::Ok, S::Error>,
 {
   let serializer = t.take_parent();
-  match f(serializer) {
+  match op(serializer) {
     Ok(value) => Ok(value),
     Err(serr) => {
-      t.capture_source(ErrorSource::Ser);
-      t.capture_error(Some(serr));
+      t.capture_error(ErrorSource::Ser, Some(serr));
       Err(de::Error::custom(TRANSLATION_FAILED))
     }
   }
@@ -379,8 +375,7 @@ where
       .take_parent()
       .serialize_seq(input.size_hint())
       .map_err(|serr| {
-        self.capture_source(ErrorSource::Ser);
-        self.capture_error(Some(serr));
+        self.capture_error(ErrorSource::Ser, Some(serr));
         de::Error::custom(TRANSLATION_FAILED)
       })?;
 
@@ -390,16 +385,14 @@ where
         Ok(None) => break,
         Ok(Some(())) => {}
         Err(derr) => {
-          self.capture_source(seed.error_source());
-          self.capture_error(seed.into_error());
+          self.capture_error(seed.error_source(), seed.into_error());
           return Err(derr);
         }
       }
     }
 
     output.end().map_err(|serr| {
-      self.capture_source(ErrorSource::Ser);
-      self.capture_error(Some(serr));
+      self.capture_error(ErrorSource::Ser, Some(serr));
       de::Error::custom(TRANSLATION_FAILED)
     })
   }
@@ -412,8 +405,7 @@ where
       .take_parent()
       .serialize_map(input.size_hint())
       .map_err(|serr| {
-        self.capture_source(ErrorSource::Ser);
-        self.capture_error(Some(serr));
+        self.capture_error(ErrorSource::Ser, Some(serr));
         de::Error::custom(TRANSLATION_FAILED)
       })?;
 
@@ -423,23 +415,20 @@ where
         Ok(None) => break,
         Ok(Some(())) => {}
         Err(derr) => {
-          self.capture_source(key_seed.0.error_source());
-          self.capture_error(key_seed.0.into_error());
+          self.capture_error(key_seed.0.error_source(), key_seed.0.into_error());
           return Err(derr);
         }
       }
 
       let value_seed = ValueSeed(Transcoder::new(&mut output));
       if let Err(derr) = input.next_value_seed(&value_seed) {
-        self.capture_source(value_seed.0.error_source());
-        self.capture_error(value_seed.0.into_error());
+        self.capture_error(value_seed.0.error_source(), value_seed.0.into_error());
         return Err(derr);
       }
     }
 
     output.end().map_err(|serr| {
-      self.capture_source(ErrorSource::Ser);
-      self.capture_error(Some(serr));
+      self.capture_error(ErrorSource::Ser, Some(serr));
       de::Error::custom(TRANSLATION_FAILED)
     })
   }
@@ -449,18 +438,17 @@ where
 /// from a deserializer to a sequence or map serializer method. As these methods
 /// require `Serialize` values, we automatically wrap the deserializer with an
 /// inner transcoder and propagate its captured error.
-fn forward_next<'de, D, F, S, E>(t: &Transcoder<S, E>, d: D, f: F) -> Result<(), D::Error>
+fn forward_next<'de, D, F, S, SErr>(t: &Transcoder<S, SErr>, d: D, op: F) -> Result<(), D::Error>
 where
   D: Deserializer<'de>,
-  F: FnOnce(S, &Transcoder<D, D::Error>) -> Result<(), E>,
+  F: FnOnce(S, &Transcoder<D, D::Error>) -> Result<(), SErr>,
 {
   let serializer = t.take_parent();
   let forwarder = Transcoder::new(d);
-  match f(serializer, &forwarder) {
+  match op(serializer, &forwarder) {
     Ok(()) => Ok(()),
     Err(serr) => {
-      t.capture_source(forwarder.error_source());
-      t.capture_error(Some(serr));
+      t.capture_error(forwarder.error_source(), Some(serr));
       match forwarder.into_error() {
         Some(derr) => Err(derr),
         None => Err(de::Error::custom(TRANSLATION_FAILED)),
@@ -528,8 +516,7 @@ where
     match deserializer.deserialize_any(&visitor) {
       Ok(value) => Ok(value),
       Err(derr) => {
-        self.capture_source(visitor.error_source());
-        self.capture_error(Some(derr));
+        self.capture_error(visitor.error_source(), Some(derr));
         match visitor.into_error() {
           Some(serr) => Err(serr),
           None => Err(ser::Error::custom(TRANSLATION_FAILED)),
@@ -585,27 +572,26 @@ impl Serialize for Value<'_> {
   where
     S: Serializer,
   {
-    use Value::*;
     match self {
-      Unit => s.serialize_unit(),
-      Bool(b) => s.serialize_bool(*b),
-      I8(n) => s.serialize_i8(*n),
-      I16(n) => s.serialize_i16(*n),
-      I32(n) => s.serialize_i32(*n),
-      I64(n) => s.serialize_i64(*n),
-      I128(n) => s.serialize_i128(*n),
-      U8(n) => s.serialize_u8(*n),
-      U16(n) => s.serialize_u16(*n),
-      U32(n) => s.serialize_u32(*n),
-      U64(n) => s.serialize_u64(*n),
-      U128(n) => s.serialize_u128(*n),
-      F32(f) => s.serialize_f32(*f),
-      F64(f) => s.serialize_f64(*f),
-      Char(c) => s.serialize_char(*c),
-      String(v) => v.serialize(s),
-      Bytes(v) => v.serialize(s),
-      Seq(v) => v.serialize(s),
-      Map(m) => {
+      Value::Unit => s.serialize_unit(),
+      Value::Bool(b) => s.serialize_bool(*b),
+      Value::I8(n) => s.serialize_i8(*n),
+      Value::I16(n) => s.serialize_i16(*n),
+      Value::I32(n) => s.serialize_i32(*n),
+      Value::I64(n) => s.serialize_i64(*n),
+      Value::I128(n) => s.serialize_i128(*n),
+      Value::U8(n) => s.serialize_u8(*n),
+      Value::U16(n) => s.serialize_u16(*n),
+      Value::U32(n) => s.serialize_u32(*n),
+      Value::U64(n) => s.serialize_u64(*n),
+      Value::U128(n) => s.serialize_u128(*n),
+      Value::F32(f) => s.serialize_f32(*f),
+      Value::F64(f) => s.serialize_f64(*f),
+      Value::Char(c) => s.serialize_char(*c),
+      Value::String(v) => v.serialize(s),
+      Value::Bytes(v) => v.serialize(s),
+      Value::Seq(v) => v.serialize(s),
+      Value::Map(m) => {
         let mut map = s.serialize_map(Some(m.len()))?;
         for (k, v) in m {
           map.serialize_entry(k, v)?;
@@ -678,7 +664,7 @@ impl<'de: 'a, 'a> Deserialize<'de> for Value<'a> {
       fn visit_seq<A: de::SeqAccess<'a>>(self, mut v: A) -> Result<Self::Value, A::Error> {
         let mut vec = Vec::with_capacity(v.size_hint().unwrap_or(0));
         while let Some(e) = v.next_element()? {
-          vec.push(e)
+          vec.push(e);
         }
         Ok(Value::Seq(vec))
       }
@@ -686,7 +672,7 @@ impl<'de: 'a, 'a> Deserialize<'de> for Value<'a> {
       fn visit_map<A: de::MapAccess<'a>>(self, mut v: A) -> Result<Self::Value, A::Error> {
         let mut vec = Vec::with_capacity(v.size_hint().unwrap_or(0));
         while let Some(entry) = v.next_entry()? {
-          vec.push(entry)
+          vec.push(entry);
         }
         Ok(Value::Map(vec))
       }
